#include "test_object.h"
#include "core/render.h"
#include "raylib.h"

START_REFLECT(TestObject);
REFLECT_TYPECLASS(TestObject, Drop);
REFLECT_TYPECLASS(TestObject, SceneNodeEntity);
REFLECT_TYPECLASS(TestObject, Renderable);
END_REFLECT(TestObject);

impl_Drop_for(TestObject,
    DestroyTestObject
)

impl_Renderable_for(TestObject,
    TestObjectDraw
)

impl_SceneNodeEntity_for(TestObject,
    TestObjectEnterTree,
    TestObjectExitTree,
    TestObjectTick
)

SceneNode *CreateTestObject() {
    TestObject *self = new(TestObject);
    return CreateSceneNode(TestObject_as_SceneNodeEntity(self));
}

void DestroyTestObject(TestObject *self) {
    free(self);
}

void TestObjectEnterTree(TestObject *self) {
    self->transform = TC_CAST(self->node->parent->entity, Transformable);
    AddRenderable(TestObject_as_Renderable(self));
}

void TestObjectExitTree(TestObject *self) {
    RemoveRenderable(TestObject_as_Renderable(self));
}

void TestObjectDraw(TestObject *self) {
}

void TestObjectTick(TestObject *self, double delta) {
    Transform local_transform = self->transform.tc->get_transform(self->transform.data);
    local_transform.rotation = QuaternionMultiply(QuaternionFromEuler(0.f, 0.01f, 0.f), local_transform.rotation);
    self->transform.tc->set_transform(self->transform.data, local_transform);
}
